@@ -52,6 +52,8 @@ inline std::string DX12ReturnValueToString(const DX12ReturnType& return_value,
                                            uint32_t              tab_count,
                                            uint32_t              tab_size)
 {
+    static_assert(nullptr == "Implement a specialisation for whatever type is instantating this base template or call "
+                             "util::ToString(<ONE_ARG>) directly if not converting a struct.");
     return util::ToString(return_value, to_string_flags, tab_count, tab_size);
 }
 
@@ -120,7 +122,7 @@ template <typename EnumType>
 inline std::string EnumPointerDecoderToString(PointerDecoder<EnumType>* pObj)
 {
     auto pDecodedObj = pObj ? pObj->GetPointer() : nullptr;
-    return pDecodedObj ? ('"' + util::ToString(*pDecodedObj) + '"') : "null";
+    return pDecodedObj ? (util::Quote(util::ToString(*pDecodedObj))) : "null";
 }
 
 template <typename DecodedType>
@@ -224,7 +226,7 @@ inline std::string EnumPointerDecoderArrayToString(const CountType&    countObj,
         tabCount,
         tabSize,
         [&]() { return pObjs && !pObjs->IsNull(); },
-        [&](size_t i) { return '"' + ToString(pObjs->GetPointer()[i], toStringFlags, tabCount + 1, tabSize) + '"'; });
+        [&](size_t i) { return util::Quote(ToString(pObjs->GetPointer()[i])); });
 }
 
 template <typename CountType, typename StructPointerDecoderType>
