@@ -54,6 +54,14 @@ void VulkanDecoderBase::DispatchStateEndMarker(uint64_t frame_number)
     }
 }
 
+void VulkanDecoderBase::DispatchFrameEndMarker(uint64_t frame_number)
+{
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessFrameEndMarker(frame_number);
+    }
+}
+
 void VulkanDecoderBase::DispatchDisplayMessageCommand(format::ThreadId thread_id, const std::string& message)
 {
     GFXRECON_UNREFERENCED_PARAMETER(thread_id);
@@ -75,6 +83,15 @@ void VulkanDecoderBase::DispatchFillMemoryCommand(
     }
 }
 
+void VulkanDecoderBase::DispatchFillMemoryResourceValueCommand(
+    const format::FillMemoryResourceValueCommandHeader& command_header, const uint8_t* data)
+{
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessFillMemoryResourceValueCommand(command_header, data);
+    }
+}
+
 void VulkanDecoderBase::DispatchResizeWindowCommand(format::ThreadId thread_id,
                                                     format::HandleId surface_id,
                                                     uint32_t         width,
@@ -130,6 +147,18 @@ void VulkanDecoderBase::DispatchDestroyHardwareBufferCommand(format::ThreadId th
     }
 }
 
+void VulkanDecoderBase::DispatchCreateHeapAllocationCommand(format::ThreadId thread_id,
+                                                            uint64_t         allocation_id,
+                                                            uint64_t         allocation_size)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(thread_id);
+
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessCreateHeapAllocationCommand(allocation_id, allocation_size);
+    }
+}
+
 void VulkanDecoderBase::DispatchSetDevicePropertiesCommand(format::ThreadId   thread_id,
                                                            format::HandleId   physical_device_id,
                                                            uint32_t           api_version,
@@ -265,6 +294,15 @@ void VulkanDecoderBase::DispatchInitImageCommand(format::ThreadId             th
     }
 }
 
+void VulkanDecoderBase::DispatchInitSubresourceCommand(const format::InitSubresourceCommandHeader& command_header,
+                                                       const uint8_t*                              data)
+{
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessInitSubresourceCommand(command_header, data);
+    }
+}
+
 size_t VulkanDecoderBase::Decode_vkUpdateDescriptorSetWithTemplate(const ApiCallInfo& call_info,
                                                                    const uint8_t*     parameter_buffer,
                                                                    size_t             buffer_size)
