@@ -23,6 +23,7 @@
 #include "decode/vulkan_export_json_consumer_base.h"
 #include "decode/custom_vulkan_struct_to_json.h"
 
+#include "util/json_util.h"
 #include "util/platform.h"
 #include "util/file_path.h"
 
@@ -30,8 +31,6 @@ GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
 using namespace util::platform;
 
-static const int kJsonIndentWidth = 2;
-
 namespace
 {
 /// @brief Convert an annotation to its string representation
@@ -149,6 +148,11 @@ void VulkanExportJsonConsumerBase::ProcessStateEndMarker(uint64_t frame_number)
     WriteStateMarkerToFile("EndMarker", frame_number);
 }
 
+void VulkanExportJsonConsumerBase::ProcessFrameEndMarker(uint64_t frame_number)
+{
+    WriteFrameMarkerToFile("EndMarker", frame_number);
+}
+
 void VulkanExportJsonConsumerBase::ProcessDisplayMessageCommand(const std::string& message)
 {
     WriteMetaCommandToFile("DisplayMessageCommand",
@@ -646,7 +650,7 @@ void VulkanExportJsonConsumerBase::WriteBlockEnd()
         FilePuts(json_options_.format == JsonFormat::JSONL ? "\n" : ",\n", file_);
     }
     // Dominates profiling (2/2):
-    const std::string block = json_data_.dump(json_options_.format == JsonFormat::JSONL ? -1 : kJsonIndentWidth);
+    const std::string block = json_data_.dump(json_options_.format == JsonFormat::JSONL ? -1 : util::kJsonIndentWidth);
     FileWriteNoLock(block.data(), sizeof(std::string::value_type), block.length(), file_);
     FileFlush(file_); /// @todo Implement a FileFlushNoLock() for all platforms.
 }
