@@ -20,6 +20,31 @@
 ** DEALINGS IN THE SOFTWARE.
 */
 
+// Define VMA_ASSERT for use in vk_mem_alloc.h
+// For debug compiles, VMA_ASSERT failure is treated as a warning.
+// For release compiles, VMA_ASSERT failure is a no-op.
+// The expr_ parameter can be in the form of 'condition && "error string"'.
+// The error string will be printed if condition is false.
+#include "util/logging.h"
+#ifdef NDEBUG
+#define VMA_ASSERT(expr_)
+#else
+#define VMA_ASSERT(expr_)                                                \
+    if (!static_cast<bool>(expr_))                                       \
+    {                                                                    \
+        std::string msg = __FILE__ ":" + std::to_string(__LINE__) + " "; \
+        if (strchr(#expr_, '"'))                                         \
+        {                                                                \
+            msg += strchr(#expr_, '"');                                  \
+        }                                                                \
+        else                                                             \
+        {                                                                \
+            msg += #expr_;                                               \
+        }                                                                \
+        GFXRECON_LOG_ERROR(msg.c_str());                                 \
+    }
+#endif
+
 // This file needs to be included first to ensure it is processed with the VMA_IMPLEMENTATION directive, in case it is
 // indirectly included by other include files.
 #define VMA_IMPLEMENTATION
@@ -384,12 +409,13 @@ void VulkanRebindAllocator::GetDeviceMemoryCommitment(VkDeviceMemory memory,
     GFXRECON_UNREFERENCED_PARAMETER(allocator_data);
 }
 
-VkResult VulkanRebindAllocator::BindBufferMemory(VkBuffer               buffer,
-                                                 VkDeviceMemory         memory,
-                                                 VkDeviceSize           memory_offset,
-                                                 ResourceData           allocator_buffer_data,
-                                                 MemoryData             allocator_memory_data,
-                                                 VkMemoryPropertyFlags* bind_memory_properties)
+VkResult VulkanRebindAllocator::BindBufferMemory(VkBuffer                                buffer,
+                                                 VkDeviceMemory                          memory,
+                                                 VkDeviceSize                            memory_offset,
+                                                 ResourceData                            allocator_buffer_data,
+                                                 MemoryData                              allocator_memory_data,
+                                                 VkMemoryPropertyFlags*                  bind_memory_properties,
+                                                 const VkPhysicalDeviceMemoryProperties& device_memory_properties)
 {
     GFXRECON_UNREFERENCED_PARAMETER(memory);
 
@@ -407,10 +433,10 @@ VkResult VulkanRebindAllocator::BindBufferMemory(VkBuffer               buffer,
 
         VmaAllocationCreateInfo create_info;
         create_info.flags = 0;
-        create_info.usage = GetBufferMemoryUsage(
-            resource_alloc_info->usage,
-            capture_memory_properties_.memoryTypes[memory_alloc_info->original_index].propertyFlags,
-            requirements);
+        create_info.usage =
+            GetBufferMemoryUsage(resource_alloc_info->usage,
+                                 device_memory_properties.memoryTypes[memory_alloc_info->original_index].propertyFlags,
+                                 requirements);
         create_info.requiredFlags  = 0;
         create_info.preferredFlags = 0;
         create_info.memoryTypeBits = 0;
@@ -550,12 +576,13 @@ VkResult VulkanRebindAllocator::BindBufferMemory2(uint32_t
     return result;
 }
 
-VkResult VulkanRebindAllocator::BindImageMemory(VkImage                image,
-                                                VkDeviceMemory         memory,
-                                                VkDeviceSize           memory_offset,
-                                                ResourceData           allocator_image_data,
-                                                MemoryData             allocator_memory_data,
-                                                VkMemoryPropertyFlags* bind_memory_properties)
+VkResult VulkanRebindAllocator::BindImageMemory(VkImage                                 image,
+                                                VkDeviceMemory                          memory,
+                                                VkDeviceSize                            memory_offset,
+                                                ResourceData                            allocator_image_data,
+                                                MemoryData                              allocator_memory_data,
+                                                VkMemoryPropertyFlags*                  bind_memory_properties,
+                                                const VkPhysicalDeviceMemoryProperties& device_memory_properties)
 {
     GFXRECON_UNREFERENCED_PARAMETER(memory);
 
@@ -576,7 +603,7 @@ VkResult VulkanRebindAllocator::BindImageMemory(VkImage                image,
         create_info.usage =
             GetImageMemoryUsage(resource_alloc_info->usage,
                                 resource_alloc_info->tiling,
-                                capture_memory_properties_.memoryTypes[memory_alloc_info->original_index].propertyFlags,
+                                device_memory_properties.memoryTypes[memory_alloc_info->original_index].propertyFlags,
                                 requirements);
         create_info.requiredFlags  = 0;
         create_info.preferredFlags = 0;
