@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 2018-2021 Valve Corporation
-** Copyright (c) 2018-2021 LunarG, Inc.
+** Copyright (c) 2018-2023 LunarG, Inc.
 ** Copyright (c) 2019 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
@@ -215,35 +215,48 @@ class VulkanCaptureManager : public CaptureManager
         EndApiCallCapture();
     }
 
+    void ProcessEndCommandApiCallCapture(VkCommandBuffer command_buffer, format::ApiCallId call_id)
+    {
+        if ((call_id == format::ApiCallId::ApiCall_vkBeginCommandBuffer) ||
+            (call_id == format::ApiCallId::ApiCall_vkResetCommandBuffer))
+        {
+            auto cmd_buffer_wrapper = GetWrapper<CommandBufferWrapper>(command_buffer);
+            GFXRECON_ASSERT(cmd_buffer_wrapper != nullptr);
+            cmd_buffer_wrapper->is_frame_boundary = false;
+        }
+    }
+
     void EndCommandApiCallCapture(VkCommandBuffer command_buffer)
     {
+        auto thread_data = GetThreadData();
+        GFXRECON_ASSERT(thread_data != nullptr);
+
         if ((GetCaptureMode() & kModeTrack) == kModeTrack)
         {
             assert(state_tracker_ != nullptr);
-
-            auto thread_data = GetThreadData();
-            assert(thread_data != nullptr);
-
             state_tracker_->TrackCommand(command_buffer, thread_data->call_id_, thread_data->parameter_buffer_.get());
         }
 
+        ProcessEndCommandApiCallCapture(command_buffer, thread_data->call_id_);
+
         EndApiCallCapture();
     }
 
     template <typename GetHandlesFunc, typename... GetHandlesArgs>
     void EndCommandApiCallCapture(VkCommandBuffer command_buffer, GetHandlesFunc func, GetHandlesArgs... args)
     {
+        auto thread_data = GetThreadData();
+        GFXRECON_ASSERT(thread_data != nullptr);
+
         if ((GetCaptureMode() & kModeTrack) == kModeTrack)
         {
             assert(state_tracker_ != nullptr);
-
-            auto thread_data = GetThreadData();
-            assert(thread_data != nullptr);
-
             state_tracker_->TrackCommand(
                 command_buffer, thread_data->call_id_, thread_data->parameter_buffer_.get(), func, args...);
         }
 
+        ProcessEndCommandApiCallCapture(command_buffer, thread_data->call_id_);
+
         EndApiCallCapture();
     }
 
@@ -886,6 +899,8 @@ class VulkanCaptureManager : public CaptureManager
     void
     PostProcess_vkQueueSubmit(VkResult result, VkQueue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence)
     {
+        PostQueueSubmit();
+
         if (((GetCaptureMode() & kModeTrack) == kModeTrack) && (result == VK_SUCCESS))
         {
             assert((state_tracker_ != nullptr) && ((submitCount == 0) || (pSubmits != nullptr)));
@@ -900,11 +915,26 @@ class VulkanCaptureManager : public CaptureManager
                                                           pSubmits[i].pSignalSemaphores);
             }
         }
+
+        // Check whether this queue submission contains a command buffer that should be treated as a frame boundary.
+        for (uint32_t i = 0; i < submitCount; ++i)
+        {
+            for (uint32_t j = 0; j < pSubmits[i].commandBufferCount; ++j)
+            {
+                auto cmd_buffer_wrapper = GetWrapper<CommandBufferWrapper>(pSubmits[i].pCommandBuffers[j]);
+                if (CheckCommandBufferWrapperForFrameBoundary(cmd_buffer_wrapper))
+                {
+                    break;
+                }
+            }
+        }
     }
 
     void PostProcess_vkQueueSubmit2(
         VkResult result, VkQueue queue, uint32_t submitCount, const VkSubmitInfo2* pSubmits, VkFence fence)
     {
+        PostQueueSubmit();
+
         if (((GetCaptureMode() & kModeTrack) == kModeTrack) && (result == VK_SUCCESS))
         {
             assert((state_tracker_ != nullptr) && ((submitCount == 0) || (pSubmits != nullptr)));
@@ -919,6 +949,20 @@ class VulkanCaptureManager : public CaptureManager
                                                               pSubmits[i].pSignalSemaphoreInfos);
             }
         }
+
+        // Check whether this queue submission contains a command buffer that should be treated as a frame boundary.
+        for (uint32_t i = 0; i < submitCount; ++i)
+        {
+            for (uint32_t j = 0; j < pSubmits[i].commandBufferInfoCount; ++j)
+            {
+                auto cmd_buffer_wrapper =
+                    GetWrapper<CommandBufferWrapper>(pSubmits[i].pCommandBufferInfos[j].commandBuffer);
+                if (CheckCommandBufferWrapperForFrameBoundary(cmd_buffer_wrapper))
+                {
+                    break;
+                }
+            }
+        }
     }
 
     void PostProcess_vkUpdateDescriptorSets(VkDevice,
@@ -1177,6 +1221,14 @@ class VulkanCaptureManager : public CaptureManager
 
     void PostProcess_vkSetLocalDimmingAMD(VkDevice device, VkSwapchainKHR swapChain, VkBool32 localDimmingEnable);
 
+    void PostProcess_vkCmdDebugMarkerInsertEXT(VkCommandBuffer                   commandBuffer,
+                                               const VkDebugMarkerMarkerInfoEXT* pMarkerInfo);
+
+    void PostProcess_vkFrameBoundaryANDROID(VkDevice device, VkSemaphore semaphore, VkImage image)
+    {
+        EndFrame();
+    }
+
 #if defined(__ANDROID__)
     void OverrideGetPhysicalDeviceSurfacePresentModesKHR(uint32_t* pPresentModeCount, VkPresentModeKHR* pPresentModes);
 #endif
@@ -1241,6 +1293,8 @@ class VulkanCaptureManager : public CaptureManager
     void ReleaseAndroidHardwareBuffer(AHardwareBuffer* hardware_buffer);
     bool CheckBindAlignment(VkDeviceSize memoryOffset);
 
+    bool CheckCommandBufferWrapperForFrameBoundary(const CommandBufferWrapper* command_buffer_wrapper);
+
   private:
     void QueueSubmitWriteFillMemoryCmd();
 
