@@ -143,9 +143,10 @@ int main(int argc, const char** argv)
             uint32_t start_frame = 0;
             uint32_t end_frame   = 0;
 
-            bool has_mfr                            = false;
-            bool quit_after_measurement_frame_range = false;
-            bool flush_measurement_frame_range      = false;
+            bool        has_mfr                            = false;
+            bool        quit_after_measurement_frame_range = false;
+            bool        flush_measurement_frame_range      = false;
+            std::string measurement_file_name;
 
             if (vulkan_replay_options.enable_vulkan)
             {
@@ -154,11 +155,17 @@ int main(int argc, const char** argv)
                 flush_measurement_frame_range      = vulkan_replay_options.flush_measurement_frame_range;
             }
 
+            if (has_mfr)
+            {
+                GetMeasurementFilename(arg_parser, measurement_file_name);
+            }
+
             gfxrecon::graphics::FpsInfo fps_info(static_cast<uint64_t>(start_frame),
                                                  static_cast<uint64_t>(end_frame),
                                                  has_mfr,
                                                  quit_after_measurement_frame_range,
-                                                 flush_measurement_frame_range);
+                                                 flush_measurement_frame_range,
+                                                 measurement_file_name);
 
             gfxrecon::decode::VulkanReplayConsumer vulkan_replay_consumer(application, vulkan_replay_options);
             gfxrecon::decode::VulkanDecoder        vulkan_decoder;
